
#ifndef VTKEYEDOMELIGHTINGVIEW_EXPORT_H
#define VTKEYEDOMELIGHTINGVIEW_EXPORT_H

#ifdef VTKEYEDOMELIGHTINGVIEW_STATIC_DEFINE
#  define VTKEYEDOMELIGHTINGVIEW_EXPORT
#  define VTKEYEDOMELIGHTINGVIEW_NO_EXPORT
#else
#  ifndef VTKEYEDOMELIGHTINGVIEW_EXPORT
#    ifdef EyeDomeLightingView_EXPORTS
        /* We are building this library */
#      define VTKEYEDOMELIGHTINGVIEW_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKEYEDOMELIGHTINGVIEW_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKEYEDOMELIGHTINGVIEW_NO_EXPORT
#    define VTKEYEDOMELIGHTINGVIEW_NO_EXPORT 
#  endif
#endif

#ifndef VTKEYEDOMELIGHTINGVIEW_DEPRECATED
#  define VTKEYEDOMELIGHTINGVIEW_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKEYEDOMELIGHTINGVIEW_DEPRECATED_EXPORT
#  define VTKEYEDOMELIGHTINGVIEW_DEPRECATED_EXPORT VTKEYEDOMELIGHTINGVIEW_EXPORT VTKEYEDOMELIGHTINGVIEW_DEPRECATED
#endif

#ifndef VTKEYEDOMELIGHTINGVIEW_DEPRECATED_NO_EXPORT
#  define VTKEYEDOMELIGHTINGVIEW_DEPRECATED_NO_EXPORT VTKEYEDOMELIGHTINGVIEW_NO_EXPORT VTKEYEDOMELIGHTINGVIEW_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKEYEDOMELIGHTINGVIEW_NO_DEPRECATED
#    define VTKEYEDOMELIGHTINGVIEW_NO_DEPRECATED
#  endif
#endif

#endif /* VTKEYEDOMELIGHTINGVIEW_EXPORT_H */
