
#ifndef VTKPVVTKEXTENSIONSEXTRACTION_EXPORT_H
#define VTKPVVTKEXTENSIONSEXTRACTION_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSEXTRACTION_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSEXTRACTION_EXPORT
#  define VTKPVVTKEXTENSIONSEXTRACTION_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSEXTRACTION_EXPORT
#    ifdef VTKExtensionsExtraction_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSEXTRACTION_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSEXTRACTION_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSEXTRACTION_NO_EXPORT
#    define VTKPVVTKEXTENSIONSEXTRACTION_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED
#  define VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSEXTRACTION_EXPORT VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSEXTRACTION_NO_EXPORT VTKPVVTKEXTENSIONSEXTRACTION_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSEXTRACTION_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSEXTRACTION_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSEXTRACTION_EXPORT_H */
