
#ifndef VTKPVVTKEXTENSIONSMISC_EXPORT_H
#define VTKPVVTKEXTENSIONSMISC_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSMISC_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSMISC_EXPORT
#  define VTKPVVTKEXTENSIONSMISC_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSMISC_EXPORT
#    ifdef VTKExtensionsMisc_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSMISC_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSMISC_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSMISC_NO_EXPORT
#    define VTKPVVTKEXTENSIONSMISC_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSMISC_DEPRECATED
#  define VTKPVVTKEXTENSIONSMISC_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSMISC_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSMISC_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSMISC_EXPORT VTKPVVTKEXTENSIONSMISC_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSMISC_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSMISC_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSMISC_NO_EXPORT VTKPVVTKEXTENSIONSMISC_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSMISC_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSMISC_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSMISC_EXPORT_H */
