
#ifndef VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT_H
#define VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSIOENSIGHT_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT
#  define VTKPVVTKEXTENSIONSIOENSIGHT_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT
#    ifdef VTKExtensionsIOEnSight_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSIOENSIGHT_NO_EXPORT
#    define VTKPVVTKEXTENSIONSIOENSIGHT_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED
#  define VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSIOENSIGHT_NO_EXPORT VTKPVVTKEXTENSIONSIOENSIGHT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSIOENSIGHT_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSIOENSIGHT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSIOENSIGHT_EXPORT_H */
